@@ -112,7 +112,6 @@ int res_init(void);
 #include "expansion_table.h"
 #undef PATCH_TABLE_EXPANSION
 
-
 static int get_config();
 static int get_environment();
 static int deadpool_init(void);
@@ -121,14 +120,6 @@ static pthread_mutex_t torsocks_init_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 void torsocks_init(void)
 {
-#define LOAD_ERROR(s,l) { \
-    const char *error; \
-    error = dlerror(); \
-    show_msg(l, "The symbol %s() was not found in any shared " \
-                     "library. The error reported was: %s!\n", s, \
-                     (error)?error:"not found"); \
-    dlerror(); \
-    }
     pthread_mutex_lock(&torsocks_init_mutex);
 
     show_msg(MSGDEBUG, "In torsocks_init \n");
@@ -150,10 +141,9 @@ void torsocks_init(void)
     dlerror();
 #ifndef USE_OLD_DLSYM
     #ifdef SUPPORT_RES_API
-    if ((realres_init = dlsym(RTLD_NEXT, "res_init")) == NULL)
-        LOAD_ERROR("res_init", MSGERR);
+      torsocks_find_library("res_init", MSGERR, realres_init);
     #endif
-    #define PATCH_TABLE_EXPANSION(e,r,s,n,b,m)  if ((real##n = dlsym(RTLD_NEXT, m)) == NULL) LOAD_ERROR(m, MSG##e);
+    #define PATCH_TABLE_EXPANSION(e,r,s,n,b,m)  torsocks_find_library(m, MSG##e, real##n);
     #include "expansion_table.h"
     #undef PATCH_TABLE_EXPANSION
 #else
@@ -243,9 +233,7 @@ static int get_config ()
 #define PATCH_TABLE_EXPANSION(e,r,s,n,b,m) \
    r n(s##SIGNATURE) { \
      if (!real##n) { \
-       dlerror(); \
-       if ((real##n = dlsym(RTLD_NEXT, m)) == NULL) \
-         LOAD_ERROR(m, MSG##e); \
+       torsocks_find_library(m, MSG##e, real##n);\
      } \
      return torsocks_##b##_guts(s##ARGNAMES, real##n); \
    }
@@ -296,6 +284,13 @@ int torsocks_connect_guts(CONNECT_SIGNATURE, int (*original_connect)(CONNECT_SIG
         return -1;
     }
 
+    /* If this is an INET6, we'll refuse it. */
+    if ((connaddr->sin_family == AF_INET6)) {
+       show_msg(MSGERR, "connect: Connection is IPv6: rejecting.\n");
+       errno = EAFNOSUPPORT;
+       return -1;
+    }
+
     /* If this isn't an INET socket we can't  */
     /* handle it, just call the real connect now        */
     if ((connaddr->sin_family != AF_INET)) {
@@ -856,8 +851,9 @@ int res_init(void)
 {
     int rc;
 
-    if (!realres_init && ((realres_init = dlsym(RTLD_NEXT, "res_init")) == NULL))
-        LOAD_ERROR("res_init", MSGERR);
+    if (!realres_init) {
+      torsocks_find_library("res_init", MSGERR, realres_init);
+    }
 
     show_msg(MSGTEST, "Got res_init request\n");
 
@@ -877,8 +873,9 @@ int EXPAND_GUTS_NAME(res_query)(RES_QUERY_SIGNATURE, int (*original_res_query)(R
 {
     int rc;
 
-    if (!original_res_query && ((original_res_query = dlsym(RTLD_NEXT, "res_query")) == NULL))
-        LOAD_ERROR("res_query", MSGERR);
+    if (!original_res_query) {
+      torsocks_find_library("res_query", MSGERR, original_res_query);
+    }
 
     show_msg(MSGTEST, "Got res_query request\n");
 
@@ -902,9 +899,9 @@ int EXPAND_GUTS_NAME(res_querydomain)(RES_QUERYDOMAIN_SIGNATURE, int (*original_
 {
     int rc;
 
-    if (!original_res_querydomain &&
-        ((original_res_querydomain = dlsym(RTLD_NEXT, "res_querydomain")) == NULL))
-        LOAD_ERROR("res_querydoimain", MSGERR);
+    if (!original_res_querydomain) {
+      torsocks_find_library("res_querydomain", MSGERR, original_res_querydomain);
+    }
 
     show_msg(MSGDEBUG, "Got res_querydomain request\n");
 
@@ -928,9 +925,9 @@ int EXPAND_GUTS_NAME(res_search)(RES_SEARCH_SIGNATURE, int (*original_res_search
 {
     int rc;
 
-    if (!original_res_search &&
-        ((original_res_search = dlsym(RTLD_NEXT, "res_search")) == NULL))
-            LOAD_ERROR("res_search", MSGERR);
+    if (!original_res_search) {
+      torsocks_find_library("res_search", MSGERR, original_res_search);
+    }
 
     show_msg(MSGTEST, "Got res_search request\n");
 
@@ -954,8 +951,9 @@ int EXPAND_GUTS_NAME(res_send)(RES_SEND_SIGNATURE, int (*original_res_send)(RES_
 {
     int rc;
 
-    if (!original_res_send && ((original_res_send = dlsym(RTLD_NEXT, "res_send")) == NULL))
-            LOAD_ERROR("res_send", MSGERR);
+    if (!original_res_send) {
+      torsocks_find_library("res_send", MSGERR, original_res_send);
+    }
 
     show_msg(MSGTEST, "Got res_send request\n");
 
